//
//
//	NFF file parser class
//
//											(C) 1997 JoOl


#include "ParserNff.h"



ParserNff::ParserNff(bool V, int32 BufLen, int32 LexmLen): ParserText(V)
{
	Lexr = new LexerNff(BufLen, LexmLen);
}


ParserNff::~ParserNff()
{
	delete Lexr;
}


bool
ParserNff::CheckType()
{
	ASSERT(ParseView());

	return true;
}


bool
ParserNff::Parse()
{
	// Always starts with a viewing system description
	ASSERT(ParseView());

	for (;;)
	{
		if (Peek(LX_CHAR_EOF))
			break;

		if (Peek(LX_TOKEN_LIGHT_LOCATION))
		{
			ASSERT(ParseLight());
		}
		else if (Peek(LX_TOKEN_BACKGROUND_COLOR))
		{
			ASSERT(ParseColor());
		}
		else if (Peek(LX_TOKEN_MATERIAL_PROPERTIES))
		{
			ASSERT(ParseMaterial());
		}
		else if (Peek(LX_TOKEN_CYLINDER))
		{
			ASSERT(ParseCylinder());
		}
		else if (Peek(LX_TOKEN_SPHERE))
		{
			ASSERT(ParseSphere());
		}
		else if (Peek(LX_TOKEN_POLYGON))
		{
			ASSERT(ParsePolygon());
		}
		else if (Peek(LX_TOKEN_POLYGON_PATCH))
		{
			ASSERT(ParsePatch());
		}
		else
		{
			// Error
			printf("lexeme  %d '%s' ?\n", ParserToken, Lexr->GetTokenLexeme(ParserToken));
			return false;
		}
	}

	Terminate();

	printf("Done...\n");
	return true;
}


bool
ParserNff::ParseVector3f(float& x, float& y, float& z)
{
	ASSERT(ParseFloat(x));
	ASSERT(ParseFloat(y));
	ASSERT(ParseFloat(z));
	return true;
}


bool
ParserNff::ParseView()
{
	float				x, y, z;
	float				angle;
	float				hither;
	uint32				ResoX, ResoY;

	MATCH(LX_TOKEN_VIEW);

	MATCH(LX_TOKEN_FROM);
	ASSERT(ParseVector3f(x, y, z));
	PRINT3("from %f %f %f\n", x, y, z);

	MATCH(LX_TOKEN_AT);
	ASSERT(ParseVector3f(x, y, z));
	PRINT3("at %f %f %f\n", x, y, z);

	MATCH(LX_TOKEN_UP);
	ASSERT(ParseVector3f(x, y, z));
	PRINT3("up %f %f %f\n", x, y, z);

	MATCH(LX_TOKEN_ANGLE);
	ASSERT(ParseFloat(angle));

	MATCH(LX_TOKEN_HITHER);
	ASSERT(ParseFloat(hither));

	MATCH(LX_TOKEN_RESOLUTION);
	ASSERT(LexInt(ResoX));
	ASSERT(LexInt(ResoY));
	PRINT4("angle=%f hither=%f reso %dx%d\n", angle, hither, ResoX, ResoY);

	return true;
}


bool
ParserNff::ParseLight()
{
	float				x, y, z;

	MATCH(LX_TOKEN_LIGHT_LOCATION);
	ASSERT(ParseVector3f(x, y, z));
	PRINT3("light %f %f %f\n", x, y, z);

	if (Peek(LX_TYPE_FLOAT))
	{
		float			r, g, b;
		ASSERT(ParseVector3f(r, g, b));
		PRINT3("light color %5.3f %5.3f %5.3f\n", r, g, b);
	}

	return true;
}


bool
ParserNff::ParseColor()
{
	float				r, g, b;

	MATCH(LX_TOKEN_BACKGROUND_COLOR);
	ASSERT(ParseVector3f(r, g, b));
	PRINT3("background color %5.3f %5.3f %5.3f\n", r, g, b);

	return true;
}


bool
ParserNff::ParseMaterial()
{
	float				r, g, b;
	float				Kd, Ks;
	float				Shine, T;
	float				RefractIndex;

	MATCH(LX_TOKEN_MATERIAL_PROPERTIES);
	ASSERT(ParseVector3f(r, g, b));
	ASSERT(ParseFloat(Kd));
	ASSERT(ParseFloat(Ks));
	ASSERT(ParseFloat(Shine));
	ASSERT(ParseFloat(T));
	ASSERT(ParseFloat(RefractIndex));
	PRINT8("material %5.3f %5.3f %5.3f, Kd=%5.3f Ks=%5.3f Shine=%5.3f T=%5.3f refract=%5.3f\n",
			r, g, b, Kd, Ks, Shine, T, RefractIndex);

	return true;
}


bool
ParserNff::ParseCylinder()
{
	float				bx, by, bz;
	float				ax, ay, az;

	MATCH(LX_TOKEN_CYLINDER);
	ASSERT(ParseVector3f(bx, by, bz));	// Base
	ASSERT(ParseVector3f(ax, ay, az));	// Apex
	PRINT6("cylinder base=(%f %f %f) apex=(%f %f %f)\n", bx, by, bz, ax, ay, az);

	return true;
}


bool
ParserNff::ParseSphere()
{
	float				x, y, z, r;
	MATCH(LX_TOKEN_SPHERE);
	ASSERT(ParseVector3f(x, y, z));
	ASSERT(ParseFloat(r));
	PRINT4("sphere center=(%f %f %f), r=%f\n", x, y, z, r);

	return true;
}


bool
ParserNff::ParsePolygon()
{
	uint32				NbVert;
	float				x, y, z;
	float				nx, ny, nz;

	MATCH(LX_TOKEN_POLYGON);
	ASSERT(LexInt(NbVert));
	PRINT1("poly %d\n", NbVert);

	for (uint32 i = 0; i < NbVert; i++)
	{
		uint32			Ln = Lexr->GetLineNumber();
		ASSERT(ParseVector3f(x, y, z));
		if ((Peek(LX_TYPE_FLOAT)) && (Ln == Lexr->GetLineNumber()))
		{
			ASSERT(ParseVector3f(nx, ny, nz));
			PRINT6("(%f %f %f) n=(%f %f %f)\n", x, y, z, nx, ny, nz);
		}
		else
		{
			PRINT3("(%f %f %f)\n", x, y, z);
		}
	}
	return true;
}


bool
ParserNff::ParsePatch()
{
	uint32				NbVert;
	float				x, y, z;
	float				nx, ny, nz;

	MATCH(LX_TOKEN_POLYGON_PATCH);
	ASSERT(LexInt(NbVert));
	PRINT1("patch %d\n", NbVert);

	for (uint32 i = 0; i < NbVert; i++)
	{
		ASSERT(ParseVector3f(x, y, z));
		if (Peek(LX_TYPE_FLOAT) || Peek(LX_TYPE_INTEGER))
		{
			ASSERT(ParseVector3f(nx, ny, nz));
			PRINT6("(%f %f %f) n=(%f %f %f)\n", x, y, z, nx, ny, nz);
		}
		else
		{
			PRINT3("(%f %f %f)\n", x, y, z);
		}
	}
	return true;
}

